@@ -3,69 +3,73 @@ use 5.006;
 use strict;
 use warnings FATAL => 'all';
 
-our $VERSION = '0.004000_02';
+our $VERSION = '0.003000';
 $VERSION = eval $VERSION;
 
 use Carp ();
+use overload ();
+use Scalar::Util qw(blessed refaddr);
 use Symbol ();
-use Devel::Confess::_Util qw(blessed refaddr weaken longmess);
-
-# detect -d:Confess.  disable debugger features for now.  we'll
-# enable them when we need them.
-if (!defined &DB::DB && $^P & 0x02) {
-  $^P = 0;
+BEGIN {
+  # fake weaken if it isn't available.  will cause leaks, but this
+  # is a brute force debugging tool, so we can deal with it.
+  *weaken = defined &Scalar::Util::weaken
+    ? \&Scalar::Util::weaken : sub ($) { 0 };
+}
+BEGIN {
+  *_CARP_DOT = $Carp::VERSION && $Carp::VERSION >= 1.25 ? sub () {1} : sub () {0};
+  if ($Carp::VERSION) {
+    $Carp::Internal{+__PACKAGE__}++;
+    *_longmess = \&Carp::longmess;
+  }
+  else {
+    *_longmess = sub {
+      my $level = 0;
+      $level++ while ((caller($level))[0] =~ /^Carp(?:::|$)/);
+      local $Carp::CarpLevel = $Carp::CarpLevel + $level;
+      &Carp::longmess;
+    }
+  }
 }
-
-$Carp::Internal{+__PACKAGE__}++;
 
 our %NoTrace;
 $NoTrace{'Throwable::Error'}++;
 $NoTrace{'Moose::Error::Default'}++;
 
-our %OPTIONS;
-
-sub _parse_options {
-  my @opts = map { /^-?(no[_-])?(.*)/; [ $_, $2, $1 ? 0 : 1 ] } @_;
-  if (!keys %OPTIONS) {
-    %OPTIONS = (
-      objects => 1,
-      builtin => undef,
-      dump => 0,
-      color => 0,
-    );
-    local $@;
-    eval { _parse_options(split ' ', $ENV{DEVEL_CONFESS_OPTIONS}||''); 1 }
-      or warn $@;
-  }
-  if (my @bad = grep { !exists $OPTIONS{$_->[1]} } @opts) {
-    Carp::croak "invalid options: " . join(', ', map { $_->[0] } @bad);
-  }
-  $OPTIONS{$_->[1]} = $_->[2]
-    for @opts;
-}
-
 my %OLD_SIG;
+my $old_verbose;
+
+my %options = (
+  objects => 1,
+  hacks => 1,
+);
 
 sub import {
   my $class = shift;
 
-  _parse_options(@_);
+  my @opts = map { /^-?(no_)?(.*)/; [ $_, $2, !$1 ] } @_;
+  if (my @bad = grep { !exists $options{$_->[1]} } @opts) {
+    Carp::croak "invalid options: " . join(', ', map { $_->[0] } @bad);
+  }
 
-  if (defined $OPTIONS{builtin}) {
-    require Devel::Confess::Builtin;
-    my $do = $OPTIONS{builtin} ? 'import' : 'unimport';
-    Devel::Confess::Builtin->$do;
+  $options{$_->[1]} = $_->[2]
+    for @opts;
+
+  if (exists $options{hacks}) {
+    require Devel::Confess::Hacks;
+    my $do = $options{hacks} ? 'import' : 'unimport';
+    Devel::Confess::Hacks->$do;
   }
 
   return
     if keys %OLD_SIG;
 
-  # enable better names for evals and anon subs
-  $^P |= 0x100 | 0x200;
-
   @OLD_SIG{qw(__DIE__ __WARN__)} = @SIG{qw(__DIE__ __WARN__)};
   $SIG{__DIE__} = \&_die;
   $SIG{__WARN__} = \&_warn;
+
+  $old_verbose = $Carp::Verbose;
+  $Carp::Verbose = 1;
 }
 
 sub _find_sig {
@@ -94,6 +98,8 @@ sub unimport {
       delete $SIG{$_};
     }
   }
+
+  $Carp::Verbose = $old_verbose;
 }
 END {
   __PACKAGE__->unimport;
@@ -105,7 +111,6 @@ sub _warn {
     $warn->(@convert);
   }
   else {
-    _colorize(\@convert, 33) if $OPTIONS{color};
     warn @convert;
   }
 }
@@ -115,47 +120,10 @@ sub _die {
     $sig->(@convert);
   }
   else {
-    _colorize(\@convert, 31) if $OPTIONS{color} && !$^S;
     die @convert;
   }
 }
 
-sub _colorize {
-  my ($convert, $color) = @_;
-  if ($ENV{DEVEL_CONFESS_FORCE_COLOR} || -t *STDERR) {
-    if (blessed $convert->[0]) {
-      if ($convert->[0]->isa('Devel::Confess::_Attached')) {
-        splice @$convert, 0, 1, _ex_as_strings($convert->[0]);
-      }
-      else {
-        $convert->[0] =~ s/(.*)/\e[${color}m$1\e[m/;
-        return;
-      }
-    }
-    $convert->[0] = "\e[${color}m$convert->[0]\e[m";
-  }
-}
-
-sub _ref_formatter {
-  require Data::Dumper;
-  local $SIG{__WARN__} = sub {};
-  local $SIG{__DIE__} = sub {};
-  no warnings 'once';
-  local $Data::Dumper::Indent = 0;
-  local $Data::Dumper::Purity = 0;
-  local $Data::Dumper::Terse = 1;
-  Data::Dumper::Dumper($_[0]);
-}
-
-sub _stack_trace {
-  no warnings 'once';
-  local $Carp::RefArgFormatter = \&_ref_formatter
-    if $OPTIONS{dump};
-  my $message = &longmess;
-  $message =~ s/\.?$/./m;
-  $message;
-}
-
 my $pack_suffix = 'A000';
 my %attached;
 
@@ -164,15 +132,15 @@ sub CLONE {
 }
 
 sub _convert {
-  __PACKAGE__->CLONE;
-  if (my $class = blessed(my $ex = $_[0])) {
+  if (my $class = blessed $_[0]) {
     return @_
-      unless $OPTIONS{objects};
+      unless $options{objects};
+    my $ex = $_[0];
     my $id = refaddr($ex);
     return @_
       if $attached{$id};
 
-    my $does = $ex->can('does') || $ex->can('DOES') || sub () { 0 };
+    my $does = $ex->can('DOES') || sub () { 0 };
     if (
       grep {
         $NoTrace{$_}
@@ -183,7 +151,8 @@ sub _convert {
       return @_;
     }
 
-    my $message = _stack_trace();
+    my $message = _longmess();
+    $message =~ s/\.?$/./m;
 
     $attached{$id} = [ $ex, $class, $message ];
     weaken $attached{$id}[0];
@@ -198,49 +167,55 @@ sub _convert {
     bless $ex, $newclass;
     $ex;
   }
-  elsif (ref($ex = $_[0])) {
+  elsif (ref(my $ex = $_[0])) {
     my $id = refaddr($ex);
     my $info = $attached{$id} ||= do {
-      my $message = _stack_trace;
+      my $message = _longmess();
+      $message =~ s/\.?$/./m;
       my $info = [ $_[0], undef, $message ];
       weaken $info->[0];
       $info;
     };
 
-    return ($^S ? @_ : ( @_, $info->[2] ));
+    return($^S ? @_ : join('', @_, $info->[2]));
   }
   elsif ((caller(1))[0] eq 'Carp') {
-    my $out = join('', @_);
-
-    my $long = longmess();
-    $out =~ s/(.*)(?:\Q$long\E| at .*? line .*?\n)\z/$1/;
-
-    return ($out, _stack_trace());
+    if (_CARP_DOT) {
+      return @_;
+    }
+    else {
+      my $message = _longmess();
+      my $out = join('', @_);
+      if ($out =~ s/\Q$message\E\z//) {
+        $message =~ s/\.?$/./m;
+      }
+      else {
+        $message =~ s/^(.*\n)//;
+        my $where = $1;
+        $where =~ s/\.?$/./m;
+        $out =~ s/(?:\Q$message\E)?\z//
+          if length $message;
+        $out .= $where;
+      }
+      $out .= $message;
+      return $out;
+    }
   }
   else {
-    my $message = _stack_trace();
+    my $message = _longmess();
     $message =~ s/^(.*\n)//;
     my $where = $1;
+    $where =~ s/\.?$/./m;
     my $out = join('', @_);
-    $out =~ s/\Q$where\E\z//;
-    return ($out, $where . $message);
+    $out =~ s/(?:\Q$where\E)?\z//;
+    $out .= $where . $message;
+    return $out;
   }
 }
 
-sub _ex_info {
+my $_ex_info = sub {
   @{$attached{refaddr $_[0]}};
-}
-sub _delete_ex_info {
-  @{ delete $attached{refaddr $_[0]} };
-}
-sub _ex_as_strings {
-  my ($ex, $class, $message) = _ex_info(@_);
-  my $newclass = ref $ex;
-  bless $ex, $class;
-  my $out = "$ex";
-  bless $ex, $newclass;
-  return ($out, $message);
-}
+};
 
 {
   package #hide
@@ -248,7 +223,7 @@ sub _ex_as_strings {
   use overload
     fallback => 1,
     'bool' => sub {
-      my ($ex, $class) = Devel::Confess::_ex_info(@_);
+      my ($ex, $class) = $_ex_info->(@_);
       my $newclass = ref $ex;
       bless $ex, $class;
       my $out = !!$ex;
@@ -256,20 +231,25 @@ sub _ex_as_strings {
       return $out;
     },
     '0+' => sub {
-      my ($ex, $class) = Devel::Confess::_ex_info(@_);
+      my ($ex, $class) = $_ex_info->(@_);
       my $newclass = ref $ex;
       bless $ex, $class;
-      my $out = 0+sprintf '%f', $ex;
+      my $out = 0+$ex;
       bless $ex, $newclass;
       return $out;
     },
     '""' => sub {
-      return join('', Devel::Confess::_ex_as_strings(@_));
+      my ($ex, $class, $message) = $_ex_info->(@_);
+      my $newclass = ref $ex;
+      bless $ex, $class;
+      my $out = "$ex" . $message;
+      bless $ex, $newclass;
+      return $out;
     },
   ;
 
   sub DESTROY {
-    my ($ex, $class) = Devel::Confess::_delete_ex_info(@_);
+    my ($ex, $class) = $_ex_info->(@_);
     my $newclass = ref $ex;
 
     Symbol::delete_package($newclass);
@@ -281,6 +261,11 @@ sub _ex_as_strings {
   }
 }
 
+# allow -d:Confess
+if (!defined &DB::DB) {
+  *DB::DB = sub {};
+}
+
 1;
 __END__
 
@@ -350,21 +335,12 @@ with no_ to disable them.
 
 =item C<objects>
 
-Enable attaching stack traces to exception objects.  Enabled by default.
+Enable attaching stack traces to exception objects.  Defaults to on.
 
-=item C<builtin>
+=item C<hacks>
 
-Load the L<Devel::Confess::Builtin> module to use built in
-stack traces on supported exception types.  Disabled by default.
-
-=item C<dump>
-
-Dumps the contents of references in arguments in stack trace, instead
-of only showing their stringified version.  Disabled by default.
-
-=item C<color>
-
-Colorizes error messages in red and warnings in yellow.  Disabled by default.
+Load the L<Devel::Confess::Hacks> module to use built in
+stack traces on supported exception types.
 
 =back
 
@@ -375,7 +351,7 @@ Colorizes error messages in red and warnings in yellow.  Disabled by default.
 Classes or roles added to this hash will not have stack traces
 attached to them.  This is useful for exception classes that provide
 their own stack traces, or classes that don't cope well with being
-re-blessed.  If L<Devel::Confess::Buildin> is loaded, it will
+re-blessed.  If L<Devel::Confess::Hacks> is loaded, it will
 automatically add its supported exception types to this hash.
 
 Default Entries:
@@ -421,10 +397,6 @@ L<Carp::Always::Color>
 
 L<Carp::Source::Always>
 
-=item *
-
-L<Carp::Always::Dump>
-
 =back
 
 Please report bugs via CPAN RT
